/*
 * #########################################################################
 * ##       Copyright © Meta Integration Technology, Inc. 1997-2022       ##
 * ##                        All Rights Reserved.                         ##
 * #########################################################################
 * ##                  http://www.metaintegration.com                     ##
 * #########################################################################
 */

var args = WScript.arguments;
var arg_version      = "";
var arg_fromFile     = "";
var arg_toFile       = "";
var arg_logFile      = "";

var validateXml = true;
var application;
var applicationVersion = 0;
var fileSystem;
fileSystem = new ActiveXObject("Scripting.FileSystemObject");

for (var index = 0; index<args.length-1; index++ ) {
	if (WScript.arguments(index)=="-version") {
		arg_version = WScript.arguments(index+1);
	}
	else if (WScript.arguments(index)=="-from") {
		arg_fromFile = WScript.arguments(index+1);
	}
	else if (WScript.arguments(index)=="-to") {
		arg_toFile = WScript.arguments(index+1);
	}
	else if (WScript.arguments(index)=="-log") {
		arg_logFile = WScript.arguments(index+1);
	}
}

// Check parameters
if (arg_version=="") {
	WScript.Echo("-version parameter is not specified.");
	usage();
	WScript.Quit(1);
}
if (arg_fromFile=="") {
    WScript.Echo("-from parameter is not specified.");
	usage();
	WScript.Quit(1);
}
if (arg_toFile=="") {
    WScript.Echo("-to parameter is not specified.");
    usage();
    WScript.Quit(1);
}
if (arg_logFile=="") {
    WScript.Echo("-log parameter is not specified.");
    usage();
    WScript.Quit(1);
}

// Print the current username
var winNetwork = new ActiveXObject("WScript.Network");
WScript.Echo("Running script as user " + winNetwork.UserName + " in domain " + winNetwork.UserDomain);

var version = 0;
if (arg_version == "9") {
    version = 9;
} else if (arg_version == "8") {
    version = 8;
} else if (arg_version == "7") {
    version = 7;
} else if (arg_version == "4") {
    version = 4;
} else {
    WScript.Echo("The specified version is not supported.");
    usage();
    WScript.Quit(1);
}

if (!fileExists(arg_fromFile)) {
    WScript.Echo("The source file does not exist.");
    usage();
    WScript.Quit(1);
}

if (!fileIsXml(arg_fromFile) && !fileIsERwin(arg_fromFile)) {
	WScript.Echo("The source file format is not supported.");
	usage();
    WScript.Quit(1);
}

if (!fileIsXml(arg_toFile) && !fileIsERwin(arg_toFile)) {
	WScript.Echo("The target file format is not supported.");
	usage();
    WScript.Quit(1);
}

if (fileExists(arg_toFile)) {
    // remove it to avoid popup warning window
    deleteFile(arg_toFile);
}

if (version==9) {
	if (fileIsXml(arg_fromFile)) {
		saveXmlAsErwin9(arg_fromFile, arg_toFile, arg_logFile, validateXml);
	} else {
		saveErwin9AsXml(arg_fromFile, arg_toFile);
	}
}
else if (version == 8) {
    if (fileIsXml(arg_fromFile)) {
        saveXmlAsErwin8(arg_fromFile, arg_toFile, arg_logFile, validateXml);
    } else {
        saveErwin8AsXml(arg_fromFile, arg_toFile);
    }
}


WScript.Echo("Script completed successfully");
WScript.Quit(0);

function usage()
{
    WScript.Echo("Usage:");
	WScript.Echo("wscript.exe ERwinXml.js -version 9 -from C:\\Temp\\eMovies9.xml -to C:\\Temp\\eMovies9.erwin -log C:\\Temp\\movie.log");
	WScript.Echo("wscript.exe ERwinXml.js -version 8 -from C:\\Temp\\eMovies8.xml -to C:\\Temp\\eMovies8.erwin -log C:\\Temp\\movie.log");
	WScript.Echo("wscript.exe ERwinXml.js -version 7 -from C:\\Temp\\eMovies7.xml -to C:\\Temp\\eMovies7.erwin -log C:\\Temp\\movie.log");
	WScript.Echo("wscript.exe ERwinXml.js -version 4 -from C:\\Temp\\eMovies4.xml -to C:\\Temp\\eMovies4.er1 -log C:\\Temp\\movie.log");

	WScript.Echo("wscript.exe ERwinXml.js -version 9 -from C:\\Temp\\eMovies9.erwin -to C:\\Temp\\eMovies9.xml -log C:\\Temp\\movie.log");
	WScript.Echo("wscript.exe ERwinXml.js -version 8 -from C:\\Temp\\eMovies8.erwin -to C:\\Temp\\eMovies8.xml -log C:\\Temp\\movie.log");
	WScript.Echo("wscript.exe ERwinXml.js -version 7 -from C:\\Temp\\eMovies7.erwin -to C:\\Temp\\eMovies7.xml -log C:\\Temp\\movie.log");
	WScript.Echo("wscript.exe ERwinXml.js -version 4 -from C:\\Temp\\eMovies4.er1 -to C:\\Temp\\eMovies4.xml -log C:\\Temp\\movie.log");
}

function fileExists(filePath)
{
    return fileSystem.FileExists(filePath);
}

function deleteFile(filePath)
{
    fileSystem.DeleteFile(filePath);
}

function getFileExtension(fileName)
{
    return fileName.split('.').pop();
}

function fileIsXml(fileName)
{
    var extension = getFileExtension(fileName).toLowerCase();
    return extension=="xml";
}

function fileIsERwin(fileName)
{
    var extension = getFileExtension(fileName).toLowerCase();
    return extension=="er1" || extension=="erwin";
}


function saveXmlAsErwin9(xmlFile, erwinFile, logFile, validateXml) {
    var comClass = "ERXML.ERXMLImport.9.0";
    WScript.Echo("Calling ERwin api " + comClass);
    try {
        this.application = new ActiveXObject(comClass);
    } catch (err) {
        WScript.Echo("Could not connect to ERwin api: " + err.message);
		WScript.Quit(1);
	}

    this.application.ImportFile(xmlFile, erwinFile, validateXml?1:2, logFile);
}

function saveErwin9AsXml(erwinFile, xmlFile) {
    var comClass = "ERXML.ERXMLExport.9.0";
    WScript.Echo("Calling ERwin api " + comClass);
    try {
        this.application = new ActiveXObject(comClass);
    } catch (err) {
        WScript.Echo("Could not connect to ERwin api: " + err.message);
		WScript.Quit(1);
	}

    this.application.ExportModel(erwinFile, xmlFile, false, 2, false);
}

function saveXmlAsErwin8(xmlFile, erwinFile, logFile, validateXml) {
    var comClass = "ERXML.ERXMLImport.8.0";
    WScript.Echo("Calling ERwin api " + comClass);
    try {
        this.application = new ActiveXObject(comClass);
    } catch (err) {
        WScript.Echo("Could not connect to ERwin api: " + err.message);
        WScript.Quit(1);
    }

    this.application.ImportFile(xmlFile, erwinFile, validateXml ? 1 : 2, logFile);
}

function saveErwin8AsXml(erwinFile, xmlFile) {
    var comClass = "ERXML.ERXMLExport.8.0";
    WScript.Echo("Calling ERwin api " + comClass);
    try {
        this.application = new ActiveXObject(comClass);
    } catch (err) {
        WScript.Echo("Could not connect to ERwin api: " + err.message);
        WScript.Quit(1);
    }

    this.application.ExportModel(erwinFile, xmlFile, false, 2, false);
}
