/*
 * #########################################################################
 * ##       Copyright  Meta Integration Technology, Inc. 1997-2017       ##
 * ##                        All Rights Reserved.                         ##
 * #########################################################################
 * ##                  http://www.metaintegration.com                     ##
 * #########################################################################
 */

// Constants
var CONFIG_VALUES_SUITE                     = "suite";
var CONFIG_VALUES_COMMON_FILES              = "commonFiles";
var CONFIG_VALUES_BINDIR                    = "bindir";
var CONFIG_VALUES_CONNECTION_SERVER_DIR     = "connectionServerDir";
var CONFIG_VALUES_BUSINESS_DESIGNER_VERSION = "businessDesignerVersion";

var args = WScript.arguments;
var bo_outputFilePath  = "";
var suiteArray = new Array(
		"HKLM\\SOFTWARE\\SAP BusinessObjects\\Suite XI 4.0",
		"HKLM\\SOFTWARE\\Business Objects\\Suite 12.0",
		"HKLM\\SOFTWARE\\Business Objects\\Suite 11.5",
		"HKLM\\SOFTWARE\\Business Objects\\Suite 11.0",
		"HKLM\\SOFTWARE\\Business Objects\\Suite 6.0"
);
var WshShell = WScript.CreateObject("WScript.Shell");

if( args.length>=1 ) {
	this.bo_outputFilePath = WScript.arguments(0);
}

// Check parameters
if( this.bo_outputFilePath=="" ) {
	WScript.Echo("please set output file path");
	usage();
	WScript.Quit(1);
}

// try to autodetect the BO installation
var suite = "";
for( var i=0; i<suiteArray.length; i++ ) {
		suite = suiteArray[i];
		try {
				WshShell.RegRead(suite + "\\default\\BusinessObjects\\BusObj Configuration\\");
				break;
		} catch (err) {
			 suite = "";
		}
}
if( suite=="" ) {
		// Could not detect the installation
		WScript.Echo("Could not detect BusinessObjects installation. Please verify the BusinessObjects client software is correctly installed.");
		WScript.Quit(1);
}

// create an XML document object
var xmlDoc = null;
try {
	xmlDoc = new ActiveXObject("Microsoft.XMLDOM");
} catch (err) {
	WScript.Echo("Can not create XML document. " + err.message);
	WScript.Quit(1);
}
xmlDoc.async = false;

// create root Node
var rootNode = xmlDoc.createElement("bo");
xmlDoc.appendChild(rootNode);

// add Suite
createElement(CONFIG_VALUES_SUITE, suite, xmlDoc, rootNode);

// add Common Files
try {
	var commonFiles = WshShell.RegRead(suite + "\\Shared\\CommonFiles");
	createElement(CONFIG_VALUES_COMMON_FILES, commonFiles, xmlDoc, rootNode);
} catch (err) { }

// add BINDIR
try {
	var bindir = WshShell.RegRead(suite + "\\Shared\\BINDIR");
	createElement(CONFIG_VALUES_BINDIR, bindir, xmlDoc, rootNode);
} catch (err) { }

// add Connection Server Directory
try {
	var connectionServerDir = WshShell.RegRead(suite + "\\default\\Shared\\ConnectionServer\\Configuration\\Config Directory");
	createElement(CONFIG_VALUES_CONNECTION_SERVER_DIR, connectionServerDir, xmlDoc, rootNode);
} catch (err) { }

// add Business Designer Version
try {
	var businessDesignerVersion = WshShell.RegRead(suite + "\\default\\BusinessObjects\\BusObj Configuration\\BusinessDesigner\\Version");
	createElement(CONFIG_VALUES_BUSINESS_DESIGNER_VERSION, businessDesignerVersion, xmlDoc, rootNode);
} catch (err) { }

var objIntro = xmlDoc.createProcessingInstruction("xml","version='1.0' encoding='UTF-8'");
xmlDoc.insertBefore(objIntro, xmlDoc.childNodes(0));
xmlDoc.save(this.bo_outputFilePath);

WScript.Echo("Script completed successfully");
WScript.Quit(0);

function usage()
{
	WScript.Echo("Specifying connection parameters: outputFilePath");
}

function createElement(elementName, elementValue, xmlDoc, parentNode)
{
	if( elementName=="" || elementValue=="" ) {
		return "";
	}
	
	var childNode = xmlDoc.createElement(elementName);
	// Add the value to the node
	childNode.appendChild(xmlDoc.createTextNode(elementValue));
	
	parentNode.appendChild(childNode);
}