/*
 * #########################################################################
 * ##       Copyright © Meta Integration Technology, Inc. 1997-2021       ##
 * ##                        All Rights Reserved.                         ##
 * #########################################################################
 * ##                  http://www.metaintegration.com                     ##
 * #########################################################################
 */

// constants
var BROWSE = "browse";
var DOWNLOAD = "download";

var args = WScript.arguments;
var bo_serverName      = "localhost";
var bo_serverPort      = "18170";
var bo_useIIS          = "False";
var bo_useSSL          = "False";
var bo_applicationName = "MartServer";
var bo_authentication  = "Server Authentication";
var bo_userName        = "";
var bo_password        = "";
var bo_paramsFilePath  = "";
var bo_outputFilePath  = "";
var bo_scapiLogFile    = "";

var bo_model           = "";	// For example: mart://Mart/Samples/eMovies
var bo_executionMode   = "";

var application;
var applicationVersion = 0;
var fileSystem;

if( args.length>=1 ) {
    bo_executionMode = WScript.arguments(0);
}
if( args.length>=4 ) {
	this.bo_paramsFilePath  = WScript.arguments(1);
	this.bo_outputFilePath  = WScript.arguments(2);
	this.bo_scapiLogFile    = WScript.arguments(3);
}
try {
	var xmlDoc = new ActiveXObject("Microsoft.XMLDOM");
	xmlDoc.async = false;
	xmlDoc.load(bo_paramsFilePath);
	
	var root = xmlDoc.documentElement;
	var rootAttributes = root.attributes;
	if( rootAttributes.length==8 ) {
		this.bo_serverName      = rootAttributes.getNamedItem("serverName").value;
		this.bo_serverPort      = rootAttributes.getNamedItem("serverPort").value;
		this.bo_useIIS          = rootAttributes.getNamedItem("useIIS").value;
		this.bo_useSSL          = rootAttributes.getNamedItem("useSSL").value;
		this.bo_applicationName = rootAttributes.getNamedItem("applicationName").value;
		this.bo_authentication  = rootAttributes.getNamedItem("authentication").value;
		this.bo_userName        = rootAttributes.getNamedItem("user").value;
		this.bo_password        = rootAttributes.getNamedItem("password").value;
	}
} catch (err) {
    WScript.Echo("Can not read a temporary XML file with connection parameters. " + err.message);
    cleanup();
	WScript.Quit(1);
}

// Check parameters
if(   (this.bo_useIIS.toLowerCase()!="true" && this.bo_useIIS.toLowerCase()!="false")
   || (this.bo_useSSL.toLowerCase()!="true" && this.bo_useSSL.toLowerCase()!="false") ) {
	WScript.Echo("Connection parameters useIIS and useSSL should have value 'true' or 'false'.");
	usage();
	WScript.Quit(1);
}
if( this.bo_outputFilePath=="" ) {
	WScript.Echo("please set output file path");
	usage();
	WScript.Quit(1);
}
if( this.bo_scapiLogFile=="" ) {
    WScript.Echo("please set temporary file path");
    usage();
    WScript.Quit(1);
}
if (bo_executionMode.toLowerCase() == DOWNLOAD ) {
	if (args.length==5) {
		this.bo_model           = WScript.arguments(4);
	}
	if( this.bo_model=="" ) {
		WScript.Echo("Connection parameter 'model' is empty");
		usage();
		WScript.Quit(1);
	}
}

// Print the current username
var winNetwork = new ActiveXObject("WScript.Network");
WScript.Echo("Running mart script as user " + winNetwork.UserName + " in domain " + winNetwork.UserDomain);

if (bo_executionMode.toLowerCase() == BROWSE) {
	///////////////////////////
	// browse for child models
	///////////////////////////
	
	// Connect to ModelMart
	connect();
	// Browse ModelMart
	browseMart();
	
} else if (bo_executionMode.toLowerCase() == DOWNLOAD) {
	//////////////
	// Download model
	//////////////

	// Connect to ModelMart
	connect();
	// Get the list of valid connection/driver types for this ERwin client version
	var connectionParameters = computeLocatorParameters();
	var listOfLocators = this.bo_model.split(";");
	if( listOfLocators.length>0 )
	{	// Save as *.erwin file
		try {
			var locator = listOfLocators[0] + "?" + connectionParameters;
			var persistenceUnit = this.application.PersistenceUnits.Add(locator, "RDO=Yes");
			persistenceUnit.Save(this.bo_outputFilePath, "OVF=Yes");
		} catch (err) {
            WScript.Echo("Could not retrieve model " + listOfLocators[0] + ". " + err.message);
			WScript.Quit(1);
		}
	}
} else {
	usage()
	WScript.Quit(1);
}

cleanup();

WScript.Echo("Script completed successfully");
WScript.Quit(0);

function usage()
{
	WScript.Echo("Specifying connection parameters: browse ServerName ServerPort UseIIS UseSSL ApplicationName Authentication UserName Password outputFilePath TemporaryLogFilePath");
	WScript.Echo("Specifying connection parameters: download ServerName ServerPort UseIIS UseSSL ApplicationName Authentication UserName Password outputFilePath TemporaryLogFilePath Model(For example: mart://Mart/Samples/eMovies)");
}

function connect()
{
	// Connect to ModelMart
	try {
		var className = "ERwin9.SCAPI";
		//WScript.Echo("Starting COM api: "+className);
		this.application = new ActiveXObject(className);
    } catch (err) {
        WScript.Echo("Could not connect to the ModelMart. " + err.message);
		WScript.Quit(1);
	}
    this.applicationVersion = this.application.Version;

    // Create temporary erwin log file
    this.fileSystem = new ActiveXObject("Scripting.FileSystemObject");
	// On Windows 10, this may require "run as administrator" rights
    this.fileSystem.CreateTextFile(this.bo_scapiLogFile, true);
    this.application.set_LogFile(this.bo_scapiLogFile, true);
}

function browseMart()
{
	//WScript.Echo("Start browse the mart.");
	var connectionParameters = computeLocatorParameters();
	var locator = computeLocator("", "", connectionParameters);
	//WScript.Echo("Connecting with locator: "+locator);
    var mart;
	try {
        mart = this.application.modelDirectories.Add(locator);
    } catch (err) {
        WScript.Echo("Could not connect to the mart. " + err.message);
	    printExtendedErrorInfo();
	    cleanup();
		WScript.Quit(1);
	}
    if( mart==null ) {
        WScript.Echo("Could not connect to the mart.");
	    printExtendedErrorInfo();
	    cleanup();
        WScript.Quit(1);
    }
	//WScript.Echo("finished application.modelDirectories.Add");

	// create an XML document object
	var xmlDoc = null;
	try {
		xmlDoc = new ActiveXObject("Microsoft.XMLDOM");
	} catch (err) {
        WScript.Echo("Can not create XML document. " + err.message);
        cleanup();
		WScript.Quit(1);
	}
	xmlDoc.async = false;
	
	var martName = mart.Name;
	//WScript.Echo("Detected mart name: "+martName);

	var martNode = xmlDoc.createElement("mart");
	// set a Name
	martNode.setAttribute("name", martName);
	
	//var martProperties = mart.PropertyBag("", true);
	//var count = martProperties.Count;
	//for (var index = 0; index<count; index++ ) {
	//	var name  = martProperties.Name(index);
	//	var value = martProperties.Value(index);
	//	WScript.Echo( "name.value: " + name + "=" + value );
	//}
	
	xmlDoc.appendChild(martNode);

	// browse for child libraries
	var libarray = [];
	var childLocator = computeLocator(mart.Name, "*", "");
	//WScript.Echo("-- LocateDirectory: " + childLocator);
	var library = mart.LocateDirectory(childLocator);
	while( library!=null ) {
		libarray.push(library);
		library = mart.LocateDirectoryNext();
	}
	for (var i=0; i<libarray.length; i++) {
		browseLibrary(mart, xmlDoc, libarray[i], martNode);
	}
	
	// browse for child models
	if( this.applicationVersion.localeCompare("9.1.0.3729")>=0 ) {
		//WScript.Echo("-- LocateDirectoryUnit: " + childLocator);
		var model = mart.LocateDirectoryUnit(childLocator);
		while( model!=null ) {
			browseModel(model, xmlDoc, martNode);
			model = mart.LocateDirectoryUnitNext();
		}
	}
	//WScript.Echo("-- browse finished----------------");
	
	var objIntro = xmlDoc.createProcessingInstruction("xml","version='1.0' encoding='UTF-8'");
	xmlDoc.insertBefore(objIntro, xmlDoc.childNodes(0));
	xmlDoc.save(this.bo_outputFilePath);
}

function getStringProperty(bag, name)
{
	var value = "";
	try {
		value = bag.Value(name);
	} catch (err) {
        WScript.Echo("An error occured when retrieving property '" + name + "'. " + err.message);
	}
	return value;
}

function pad(n) {
    return (n < 10) ? '0' + n : n;
}

function getDateProperty(bag, name) {
    var value = getStringProperty(bag, name);
    if (value == "") {
        return "";
    }

    var d = new Date(value);

    return d.getUTCFullYear()
		+ '-' + pad(d.getUTCMonth() + 1)
		+ '-' + pad(d.getUTCDate())
		+ ' ' + pad(d.getUTCHours())
		+ ':' + pad(d.getUTCMinutes())
		+ ':' + pad(d.getUTCSeconds());
}

function truncateLocator(s)
{	// just remove parameters after the '?'
	var separator = s.indexOf("?");
	if( separator!=-1 ) {
		return s.substring(0, separator);
	} else {
		return s;
	}
}

function browseLibrary(mart, xmlDoc, library, directoryNode)
{
	var bag = library.PropertyBag("", true);
	var description = getStringProperty(bag, "Description");
	var libraryLocator = getStringProperty(bag, "Locator");
	var created = getDateProperty(bag, "Created");
	var updated = getDateProperty(bag, "Updated");
	var createdBy = getStringProperty(bag, "Created_By");
	var updatedBy = getStringProperty(bag, "Updated_By");
	
	var childDirectoryNode = xmlDoc.createElement("directory");
	// set a Name
	childDirectoryNode.setAttribute("name", library.Name);
	//WScript.Echo("-- browse: " + library.Name);
	// set a Description
	childDirectoryNode.setAttribute("description", description);
	// set a Locator
	childDirectoryNode.setAttribute("locator", libraryLocator);
	childDirectoryNode.setAttribute("creationDate", created);
	childDirectoryNode.setAttribute("lastModification", updated);
	childDirectoryNode.setAttribute("createdBy", createdBy);
	childDirectoryNode.setAttribute("updatedBy", updatedBy);
	
	directoryNode.appendChild(childDirectoryNode);
	
	var libarray = [];
	var childLocator = truncateLocator(libraryLocator) + "/*";
	//WScript.Echo("-- LocateDirectory: " + childLocator);
	var childLibrary = mart.LocateDirectory(childLocator);
	while( childLibrary!=null ) {
		libarray.push(childLibrary);
		childLibrary = mart.LocateDirectoryNext();
	}
	for (var i=0; i<libarray.length; i++) {
		browseLibrary(mart, xmlDoc, libarray[i], childDirectoryNode);
	}
	
	//WScript.Echo("-- LocateDirectoryUnit: " + childLocator);
	var model = mart.LocateDirectoryUnit(childLocator);
	while( model!=null ) {
		browseModel(model, xmlDoc, childDirectoryNode);
		model = mart.LocateDirectoryUnitNext();
	}
}

function browseModel(model, xmlDoc, parentNode)
{
	var bag = model.PropertyBag("", true);
	var description = getStringProperty(bag, "Description");
	var modelLocator = getStringProperty(bag, "Locator");
	var created = getDateProperty(bag, "Created");
	var updated = getDateProperty(bag, "Updated");
	var createdBy = getStringProperty(bag, "Created_By");
	var updatedBy = getStringProperty(bag, "Updated_By");
	
	var modelNode = xmlDoc.createElement("directoryUnit");
	// set a Name
	modelNode.setAttribute("name", model.Name);
	//WScript.Echo("------ browse model : " + model.Name);
	// set a Description
	modelNode.setAttribute("description", description);
	// set a CreationDate
    modelNode.setAttribute("creationDate", created);
	// set a LastModification
	modelNode.setAttribute("lastModification", updated);
	// set a Locator
	modelNode.setAttribute("locator", modelLocator);
	modelNode.setAttribute("createdBy", createdBy);
	modelNode.setAttribute("updatedBy", updatedBy);
	parentNode.appendChild(modelNode);
}

function computeLocator(databaseName, library, extraParameters)
{
	var locator = "";
	locator += "mart://";
	locator += databaseName;
	locator += "/";
	if( library!="" ) {
		locator += library;
	}
	if( extraParameters!="" ) {
		locator += "?" + extraParameters;
	}
	return locator;
}

function computeLocatorParameters()
{
	var parameters = "";
	parameters += "SRV=" + this.bo_serverName;
	if( this.bo_useIIS.toLowerCase()=="true" ) {
		parameters += ";IIS=YES";
	} else {
		parameters += ";IIS=NO";
		parameters += ";PRT=" + this.bo_serverPort;
	}
	if( this.applicationVersion.localeCompare("9.1")>=0 ) {
		// 9.0 did not support the SSL parameter
		// it was added since 9.1
		if( this.bo_useSSL.toLowerCase()=="true" ) {
			parameters += ";SSL=YES";
		} else {
			parameters += ";SSL=NO";
		}
	}
	parameters += ";ASR=" + this.bo_applicationName;
	if( this.bo_authentication=="Server Authentication" ) {
		parameters += ";TRC=NO";
		if( this.bo_userName!="" ) {
			parameters += ";UID=" + this.bo_userName;
		}
		if( this.bo_password!="" ) {
			// ERwin fails to parse the locator if password is empty, 
			// so do not specify the PSW parameter at all, if no password is available
			parameters += ";PSW=" + this.bo_password;
		}
	} else if( this.bo_authentication=="Windows Authentication" ) {
		parameters += ";TRC=YES";
	}
	return parameters;
}

function printExtendedErrorInfo()
{
    var emptyLog = true;
    var env = this.application.ApplicationEnvironment;
    var category = "Application.Api.MessageLog";
    var logMessages = env.PropertyBag(category);

    var count = logMessages.Count;
    for (var index = 0; index<count; index++ ) {
		var name  = logMessages.Name(index);
		var value = logMessages.Value(index);
        if( name=="Is_Empty" && value==true ) {
            emptyLog = value;
		}
    }
	for( var index=0; index<count; index++ ) {
		var name  = logMessages.Name(index);
		var value = logMessages.Value(index);
		if (name == "Log") {
			printExtendedErrorInfo2(value, emptyLog);
		}
    }

    // Also check in the new log file
    if( this.fileSystem.FileExists(bo_scapiLogFile) ) {
        var f = this.fileSystem.OpenTextFile(bo_scapiLogFile, 1);
        while( !f.AtEndOfStream )
        {   // Read a line
            var sLine = f.ReadLine();
            if (sLine != "") {
                // print the message line
                WScript.Echo(sLine);
            }
        }
        f.Close();
    }
}

function printExtendedErrorInfo2(erwinLog, emptyLog)
{
/*    var logArray = erwinLog.parray;
    if (logArray != null)
    {   // this is a SAFEARRAY 
        for (var i = 1; i <= logArray.dimensions(); i++) {
            
        }
        var lowerBound = logArray.LBound;
        var upperBound = logArray.UBound;
    }*/
}

function cleanup()
{
}